@@ -9,28 +9,28 @@
 # lost the next time ruby is built.
 
 module RbConfig
-  RUBY_VERSION.start_with?("3.3.") or
-    raise "ruby lib version (3.3.10) doesn't match executable version (#{RUBY_VERSION})"
+  RUBY_VERSION.start_with?("3.4.") or
+    raise "ruby lib version (3.4.1) doesn't match executable version (#{RUBY_VERSION})"
 
   # Ruby installed directory.
-  TOPDIR = File.dirname(__FILE__).chomp!("/lib/ruby/3.3/aarch64-openbsd")
+  TOPDIR = File.dirname(__FILE__).chomp!("/lib/ruby/3.4/aarch64-openbsd")
   # DESTDIR on make install.
   DESTDIR = '' unless defined? DESTDIR
   # The hash configurations stored.
   CONFIG = {}
   CONFIG["DESTDIR"] = DESTDIR
   CONFIG["MAJOR"] = "3"
-  CONFIG["MINOR"] = "3"
-  CONFIG["TEENY"] = "10"
-  CONFIG["PATCHLEVEL"] = "183"
-  CONFIG["INSTALL"] = '/usr/obj/ports/ruby-3.3.10/bin/install -c'
+  CONFIG["MINOR"] = "4"
+  CONFIG["TEENY"] = "1"
+  CONFIG["PATCHLEVEL"] = "0"
+  CONFIG["INSTALL"] = '/usr/obj/ports/ruby-3.4.1/bin/install -c'
   CONFIG["EXEEXT"] = ""
   CONFIG["prefix"] = (TOPDIR || DESTDIR + "/usr/local")
-  CONFIG["ruby_install_name"] = "$(RUBY_BASE_NAME)33"
-  CONFIG["RUBY_INSTALL_NAME"] = "$(RUBY_BASE_NAME)33"
-  CONFIG["RUBY_SO_NAME"] = "ruby33"
+  CONFIG["ruby_install_name"] = "$(RUBY_BASE_NAME)34"
+  CONFIG["RUBY_INSTALL_NAME"] = "$(RUBY_BASE_NAME)34"
+  CONFIG["RUBY_SO_NAME"] = "ruby34"
   CONFIG["exec"] = "exec"
-  CONFIG["ruby_pc"] = "ruby-3.3.pc"
+  CONFIG["ruby_pc"] = "ruby-3.4.pc"
   CONFIG["CC_WRAPPER"] = ""
   CONFIG["PACKAGE"] = "ruby"
   CONFIG["BUILTIN_TRANSSRCS"] = " enc/trans/newline.c"
@@ -45,7 +45,7 @@ module RbConfig
   CONFIG["RUBY_SEARCH_PATH"] = ""
   CONFIG["UNIVERSAL_INTS"] = ""
   CONFIG["UNIVERSAL_ARCHNAMES"] = ""
-  CONFIG["configure_args"] = " '--disable-yjit' '--enable-shared' '--program-suffix=33' '--with-soname=ruby33' '--with-ruby-version=3.3' '--with-mantype=doc' '--with-setjmp-type=_setjmp' '--enable-pthread' '--enable-ipv6' '--without-baseruby' '--without-bundled-libffi' '--disable-option-checking' '--with-compress-debug-sections=no' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' '--disable-gtk-doc' 'CC=cc' 'CXX=c++' 'CFLAGS=-O2 -pipe -g' 'LDFLAGS=-L/usr/local/lib' 'CPPFLAGS=-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include' 'CXXFLAGS=-O2 -pipe -g'"
+  CONFIG["configure_args"] = " '--disable-yjit' '--enable-shared' '--program-suffix=34' '--with-soname=ruby34' '--with-ruby-version=3.4' '--with-mantype=doc' '--with-setjmp-type=_setjmp' '--enable-pthread' '--enable-ipv6' '--without-baseruby' '--without-bundled-libffi' '--disable-option-checking' '--with-compress-debug-sections=no' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' '--disable-gtk-doc' 'CC=cc' 'CXX=c++' 'CFLAGS=-O2 -pipe -g' 'LDFLAGS=-L/usr/local/lib' 'CPPFLAGS=-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include' 'CXXFLAGS=-O2 -pipe -g'"
   CONFIG["CONFIGURE"] = "configure"
   CONFIG["vendorarchdir"] = "$(vendorlibdir)/$(sitearch)"
   CONFIG["vendorlibdir"] = "$(vendordir)/$(ruby_version)"
@@ -55,7 +55,7 @@ module RbConfig
   CONFIG["sitedir"] = "$(rubylibprefix)/site_ruby"
   CONFIG["rubyarchdir"] = "$(rubylibdir)/$(arch)"
   CONFIG["rubylibdir"] = "$(rubylibprefix)/$(ruby_version)"
-  CONFIG["ruby_version"] = "3.3"
+  CONFIG["ruby_version"] = "3.4"
   CONFIG["sitearch"] = "$(arch)"
   CONFIG["arch"] = "aarch64-openbsd"
   CONFIG["sitearchincludedir"] = "$(includedir)/$(sitearch)"
@@ -64,7 +64,7 @@ module RbConfig
   CONFIG["archlibdir"] = "$(libdir)/$(arch)"
   CONFIG["libdirname"] = "libdir"
   CONFIG["RUBY_EXEC_PREFIX"] = "/usr/local"
-  CONFIG["RUBY_LIB_VERSION"] = "3.3"
+  CONFIG["RUBY_LIB_VERSION"] = "3.4"
   CONFIG["RUBY_LIB_VERSION_STYLE"] = ""
   CONFIG["RI_BASE_NAME"] = "ri"
   CONFIG["ridir"] = "$(datarootdir)/$(RI_BASE_NAME)"
@@ -97,6 +97,7 @@ module RbConfig
   CONFIG["rubyw_install_name"] = ""
   CONFIG["EXTDLDFLAGS"] = ""
   CONFIG["EXTLDFLAGS"] = ""
+  CONFIG["hardenflags"] = "-mbranch-protection=standard -fstack-protector-strong -U_FORTIFY_SOURCE -D_FORTIFY_SOURCE=2"
   CONFIG["strict_warnflags"] = ""
   CONFIG["warnflags"] = "-Wall -Wextra -Wextra-tokens -Wdeprecated-declarations -Wdivision-by-zero -Wdiv-by-zero -Wimplicit-function-declaration -Wimplicit-int -Wpointer-arith -Wshorten-64-to-32 -Wwrite-strings -Wold-style-definition -Wmissing-noreturn -Wno-cast-function-type -Wno-constant-logical-operand -Wno-long-long -Wno-missing-field-initializers -Wno-overlength-strings -Wno-parentheses-equality -Wno-self-assign -Wno-tautological-compare -Wno-unused-parameter -Wno-unused-value -Wunused-variable -Wmisleading-indentation -Wundef"
   CONFIG["debugflags"] = "-ggdb3"
@@ -112,6 +113,7 @@ module RbConfig
   CONFIG["CARGO"] = ""
   CONFIG["RUSTC"] = "no"
   CONFIG["INSTALL_STATIC_LIBRARY"] = "no"
+  CONFIG["modular_gc_dir"] = ""
   CONFIG["EXECUTABLE_EXTS"] = ""
   CONFIG["ARCHFILE"] = ""
   CONFIG["LIBRUBY_RELATIVE"] = "no"
@@ -125,9 +127,9 @@ module RbConfig
   CONFIG["TRY_LINK"] = ""
   CONFIG["PRELOADENV"] = "LD_PRELOAD"
   CONFIG["LIBPATHENV"] = "LD_LIBRARY_PATH"
-  CONFIG["RPATHFLAG"] = " -Wl,-rpath,%1$-s"
-  CONFIG["LIBPATHFLAG"] = " -L%1$-s"
-  CONFIG["LINK_SO"] = ""
+  CONFIG["RPATHFLAG"] = "-Wl,-rpath,%1$-s"
+  CONFIG["LIBPATHFLAG"] = "-L%1$-s"
+  CONFIG["LINK_SO"] = "\n$(POSTLINK)"
   CONFIG["ADDITIONAL_DLDFLAGS"] = ""
   CONFIG["ENCSTATIC"] = ""
   CONFIG["EXTSTATIC"] = ""
@@ -160,17 +162,16 @@ module RbConfig
   CONFIG["DOXYGEN"] = ""
   CONFIG["DOT"] = ""
   CONFIG["MKDIR_P"] = "mkdir -p"
-  CONFIG["INSTALL_DATA"] = "/usr/obj/ports/ruby-3.3.10/bin/install -c -m 644"
-  CONFIG["INSTALL_SCRIPT"] = "/usr/obj/ports/ruby-3.3.10/bin/install -c -m 755"
-  CONFIG["INSTALL_PROGRAM"] = "/usr/obj/ports/ruby-3.3.10/bin/install -c  -m 755"
+  CONFIG["INSTALL_DATA"] = "/usr/obj/ports/ruby-3.4.1/bin/install -c -m 644"
+  CONFIG["INSTALL_SCRIPT"] = "/usr/obj/ports/ruby-3.4.1/bin/install -c -m 755"
+  CONFIG["INSTALL_PROGRAM"] = "/usr/obj/ports/ruby-3.4.1/bin/install -c  -m 755"
   CONFIG["SET_MAKE"] = ""
   CONFIG["LN_S"] = "ln -s"
   CONFIG["DLLWRAP"] = ""
   CONFIG["WINDRES"] = ""
-  CONFIG["ASFLAGS"] = "-mbranch-protection=standard"
   CONFIG["ARFLAGS"] = "rcD "
   CONFIG["try_header"] = ""
-  CONFIG["CC_VERSION_MESSAGE"] = "OpenBSD clang version 19.1.7\nTarget: aarch64-unknown-openbsd7.8\nThread model: posix\nInstalledDir: /usr/bin"
+  CONFIG["CC_VERSION_MESSAGE"] = "OpenBSD clang version 16.0.6\nTarget: aarch64-unknown-openbsd7.6\nThread model: posix\nInstalledDir: /usr/bin"
   CONFIG["CC_VERSION"] = "$(CC) --version"
   CONFIG["CSRCFLAG"] = ""
   CONFIG["COUTFLAG"] = "-o "
@@ -184,19 +185,20 @@ module RbConfig
   CONFIG["CPPFLAGS"] = "-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include $(DEFS) $(cppflags)"
   CONFIG["LDFLAGS"] = "-L. -L/usr/local/lib -fstack-protector-strong -Wl,-E -Wl,--no-as-needed"
   CONFIG["CFLAGS"] = "-O2 -pipe -g $(cflags) -fPIC"
-  CONFIG["STRIP"] = "strip -S -x"
+  CONFIG["STRIP"] = "strip -A -n"
   CONFIG["RANLIB"] = "ranlib"
   CONFIG["OBJDUMP"] = "objdump"
   CONFIG["OBJCOPY"] = "objcopy"
   CONFIG["NM"] = "nm"
   CONFIG["LD"] = "ld"
   CONFIG["CXX"] = "c++"
-  CONFIG["AS"] = "gas"
+  CONFIG["AS"] = ""
   CONFIG["AR"] = "ar"
   CONFIG["CC"] = "cc"
   CONFIG["wasmoptflags"] = ""
   CONFIG["WASMOPT"] = ""
-  CONFIG["target_os"] = "openbsd7.8"
+  CONFIG["config_target"] = "aarch64-unknown-openbsd7.6"
+  CONFIG["target_os"] = "openbsd7.6"
   CONFIG["target_vendor"] = "unknown"
   CONFIG["target_cpu"] = "aarch64"
   CONFIG["target"] = "$(target_cpu)-$(target_vendor)-$(target_os)"
@@ -204,10 +206,10 @@ module RbConfig
   CONFIG["host_vendor"] = "$(target_vendor)"
   CONFIG["host_cpu"] = "$(target_cpu)"
   CONFIG["host"] = "$(target)"
-  CONFIG["build_os"] = "openbsd7.8"
+  CONFIG["build_os"] = "openbsd7.6"
   CONFIG["build_vendor"] = "unknown"
   CONFIG["build_cpu"] = "aarch64"
-  CONFIG["build"] = "aarch64-unknown-openbsd7.8"
+  CONFIG["build"] = "aarch64-unknown-openbsd7.6"
   CONFIG["RUBY_VERSION_NAME"] = "$(RUBY_BASE_NAME)-$(ruby_version)"
   CONFIG["RUBYW_BASE_NAME"] = "rubyw"
   CONFIG["RUBY_BASE_NAME"] = "ruby"
@@ -217,7 +219,7 @@ module RbConfig
   CONFIG["GIT"] = "git"
   CONFIG["cxxflags"] = ""
   CONFIG["cppflags"] = ""
-  CONFIG["cflags"] = "-fdeclspec $(optflags) $(debugflags) $(warnflags)"
+  CONFIG["cflags"] = "$(hardenflags) -fdeclspec  $(optflags) $(debugflags) $(warnflags)"
   CONFIG["MAKEDIRS"] = "mkdir -p"
   CONFIG["target_alias"] = ""
   CONFIG["host_alias"] = "$(target_alias)"
@@ -247,7 +249,7 @@ module RbConfig
   CONFIG["libexecdir"] = "$(exec_prefix)/libexec"
   CONFIG["sbindir"] = "$(exec_prefix)/sbin"
   CONFIG["bindir"] = "$(exec_prefix)/bin"
-  CONFIG["program_transform_name"] = "s&$$&33&;s&^&&"
+  CONFIG["program_transform_name"] = "s&$$&34&;s&^&&"
   CONFIG["exec_prefix"] = "$(prefix)"
   CONFIG["PACKAGE_URL"] = ""
   CONFIG["PACKAGE_BUGREPORT"] = ""
