@@ -8,14 +8,26 @@ module Bundler
     include MatchPlatform
     include ForcePlatform
 
-    attr_reader :name, :version, :platform
+    attr_reader :name, :version, :platform, :materialization
     attr_accessor :source, :remote, :force_ruby_platform, :dependencies, :required_ruby_version, :required_rubygems_version
 
+    #
+    # For backwards compatibility with existing lockfiles, if the most specific
+    # locked platform is not a specific platform like x86_64-linux or
+    # universal-java-11, then we keep the previous behaviour of resolving the
+    # best platform variant at materiliazation time. For previous bundler
+    # versions (before 2.2.0) this was always the case (except when the lockfile
+    # only included non-ruby platforms), but we're also keeping this behaviour
+    # on newer bundlers unless users generate the lockfile from scratch or
+    # explicitly add a more specific platform.
+    #
+    attr_accessor :most_specific_locked_platform
+
     alias_method :runtime_dependencies, :dependencies
 
     def self.from_spec(s)
       lazy_spec = new(s.name, s.version, s.platform, s.source)
-      lazy_spec.dependencies = s.dependencies
+      lazy_spec.dependencies = s.runtime_dependencies
       lazy_spec.required_ruby_version = s.required_ruby_version
       lazy_spec.required_rubygems_version = s.required_rubygems_version
       lazy_spec
@@ -27,9 +39,26 @@ module Bundler
       @dependencies  = []
       @required_ruby_version = Gem::Requirement.default
       @required_rubygems_version = Gem::Requirement.default
-      @platform      = platform || Gem::Platform::RUBY
-      @source        = source
+      @platform = platform || Gem::Platform::RUBY
+
+      @original_source = source
+      @source = source
+
       @force_ruby_platform = default_force_ruby_platform
+      @most_specific_locked_platform = nil
+      @materialization = nil
+    end
+
+    def missing?
+      @materialization == self
+    end
+
+    def incomplete?
+      @materialization.nil?
+    end
+
+    def source_changed?
+      @original_source != source
     end
 
     def full_name
@@ -92,16 +121,31 @@ module Bundler
       out
     end
 
+    def materialize_strictly
+      source.local!
+
+      matching_specs = source.specs.search(self)
+      return self if matching_specs.empty?
+
+      __materialize__(matching_specs)
+    end
+
+    def materialized_for_installation
+      @materialization = materialize_for_installation
+
+      self unless incomplete?
+    end
+
     def materialize_for_installation
       source.local!
 
-      matching_specs = source.specs.search(use_exact_resolved_specifications? ? self : [name, version])
-      return self if matching_specs.empty?
-
-      candidates = if use_exact_resolved_specifications?
-        matching_specs
+      if use_exact_resolved_specifications?
+        materialize_strictly
       else
-        target_platform = ruby_platform_materializes_to_ruby_platform? ? platform : local_platform
+        matching_specs = source.specs.search([name, version])
+        return self if matching_specs.empty?
+
+        target_platform = source.is_a?(Source::Path) ? platform : local_platform
 
         installable_candidates = GemHelpers.select_best_platform_match(matching_specs, target_platform)
 
@@ -112,10 +156,8 @@ module Bundler
           installable_candidates = GemHelpers.select_best_platform_match(matching_specs, platform)
         end
 
-        installable_candidates
+        __materialize__(installable_candidates)
       end
-
-      __materialize__(candidates)
     end
 
     # If in frozen mode, we fallback to a non-installable candidate because by
@@ -129,12 +171,28 @@ module Bundler
       end
       if search.nil? && fallback_to_non_installable
         search = candidates.last
-      else
-        search.dependencies = dependencies if search && search.full_name == full_name && (search.is_a?(RemoteSpecification) || search.is_a?(EndpointSpecification))
+      elsif search && search.full_name == full_name
+        # We don't validate locally installed dependencies but accept what's in
+        # the lockfile instead for performance, since loading locally installed
+        # dependencies would mean evaluating all gemspecs, which would affect
+        # `bundler/setup` performance
+        if search.is_a?(StubSpecification)
+          search.dependencies = dependencies
+        else
+          if !source.is_a?(Source::Path) && search.runtime_dependencies.sort != dependencies.sort
+            raise IncorrectLockfileDependencies.new(self)
+          end
+
+          search.locked_platform = platform if search.instance_of?(RemoteSpecification) || search.instance_of?(EndpointSpecification)
+        end
       end
       search
     end
 
+    def inspect
+      "#<#{self.class} @name=\"#{name}\" (#{full_name.delete_prefix("#{name}-")})>"
+    end
+
     def to_s
       lock_name
     end
@@ -151,23 +209,13 @@ module Bundler
     private
 
     def use_exact_resolved_specifications?
-      @use_exact_resolved_specifications ||= !source.is_a?(Source::Path) && ruby_platform_materializes_to_ruby_platform?
+      !source.is_a?(Source::Path) && ruby_platform_materializes_to_ruby_platform?
     end
 
-    #
-    # For backwards compatibility with existing lockfiles, if the most specific
-    # locked platform is not a specific platform like x86_64-linux or
-    # universal-java-11, then we keep the previous behaviour of resolving the
-    # best platform variant at materiliazation time. For previous bundler
-    # versions (before 2.2.0) this was always the case (except when the lockfile
-    # only included non-ruby platforms), but we're also keeping this behaviour
-    # on newer bundlers unless users generate the lockfile from scratch or
-    # explicitly add a more specific platform.
-    #
     def ruby_platform_materializes_to_ruby_platform?
       generic_platform = generic_local_platform == Gem::Platform::JAVA ? Gem::Platform::JAVA : Gem::Platform::RUBY
 
-      !Bundler.most_specific_locked_platform?(generic_platform) || force_ruby_platform || Bundler.settings[:force_ruby_platform]
+      (most_specific_locked_platform != generic_platform) || force_ruby_platform || Bundler.settings[:force_ruby_platform]
     end
   end
 end
